#ifndef StatusManager_define
#define StatusManager_define 1
#include "utiles.h"
#define FLAG_SD 0x01 //Status ok sd
#define FLAG_GPS 0x02 //Status ok gps
#define FLAG_SMS 0x04 //Status ok sms
#define TIMER_REPORTE_STATUS 3000//cada 3 seg
#define TIMER_ON_LED 200
class StatusManager
{
    int flagStatus;
    unsigned long timerStatus;
    int ledSD;
    int ledGPS;
    int ledSMS;
  public:
    StatusManager(int pinLedSD = -1, int pinLedGPS = -1, int pinLedSMS = -1);
    void EmpiezaInicio();
    void FinInicio();
    void SetStatusSD(bool ok);

    void SetStatusGPS(bool ok);

    void SetStatusSMS(bool ok);

    void Procesa();


  private:

    bool GetStatusSD();
    bool GetStatusGPS();
    bool GetStatusSMS();
    void parpadeoLed(int pled, bool enciende);

};
#endif
